/* vim:set ts=2 sw=2 sts=2 et: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

/* Toolbar and Toolbar items */

.devtools-toolbar {
  -moz-appearance: none;
  padding: 4px 3px;
  box-shadow: 0 1px 0 0 hsla(210, 16%, 76%, .2) inset;
  background: -moz-linear-gradient(top, hsl(210,11%,36%), hsl(210,11%,18%));
  color: hsl(210,30%,85%);
}

.devtools-menulist,
.devtools-toolbarbutton {
  -moz-appearance: none;
  -moz-box-align: center;
  min-width: 78px;
  min-height: 22px;
  text-shadow: 0 -1px 0 hsla(210,8%,5%,.45);
  border: 1px solid hsla(210,8%,5%,.45);
  border-radius: 3px;
  background: -moz-linear-gradient(hsla(212,7%,57%,.35), hsla(212,7%,57%,.1)) padding-box;
  box-shadow: 0 1px 0 hsla(210,16%,76%,.15) inset, 0 0 0 1px hsla(210,16%,76%,.15) inset, 0 1px 0 hsla(210,16%,76%,.15);
  margin: 0 3px;
  color: inherit;
}

.devtools-toolbarbutton > .toolbarbutton-menubutton-button {
  -moz-box-orient: horizontal;
}

.devtools-menulist:-moz-focusring,
.devtools-toolbarbutton:-moz-focusring {
  outline: 1px dotted hsla(210,30%,85%,0.4);
  outline-offset: -4px;
}

.devtools-toolbarbutton:not([label]) {
  min-width: 32px;
}

.devtools-toolbarbutton:not([checked=true]):hover:active {
  border-color: hsla(210,8%,5%,.6);
  background: -moz-linear-gradient(hsla(220,6%,10%,.3), hsla(212,7%,57%,.15) 65%, hsla(212,7%,57%,.3));
  box-shadow: 0 0 3px hsla(210,8%,5%,.25) inset, 0 1px 3px hsla(210,8%,5%,.25) inset, 0 1px 0 hsla(210,16%,76%,.15);
}

.devtools-menulist[open=true],
.devtools-toolbarbutton[open=true],
.devtools-toolbarbutton[checked=true] {
  border-color: hsla(210,8%,5%,.6) !important;
  background: -moz-linear-gradient(hsla(220,6%,10%,.6), hsla(210,11%,18%,.45) 75%, hsla(210,11%,30%,.4));
  box-shadow: 0 1px 3px hsla(210,8%,5%,.25) inset, 0 1px 3px hsla(210,8%,5%,.25) inset, 0 1px 0 hsla(210,16%,76%,.15);
}

.devtools-toolbarbutton[checked=true] {
  color: hsl(208,100%,60%) !important;
}

.devtools-toolbarbutton[checked=true]:hover {
  background-color: transparent !important;
}

.devtools-toolbarbutton[checked=true]:hover:active {
  background-color: hsla(210,8%,5%,.2) !important;
}

.devtools-menulist > .menulist-label-box {
  text-align: center;
}

.devtools-menulist > .menulist-dropmarker {
  -moz-appearance: none;
  display: -moz-box;
  list-style-image: url("chrome://browser/skin/devtools/dropmarker.png");
  -moz-box-align: center;
  min-width: 16px;
}

.devtools-toolbarbutton[type=menu-button] > .toolbarbutton-menubutton-button {
  -moz-appearance: none;
  color: inherit;
  border-width: 0;
  -moz-border-end: 1px solid hsla(210,8%,5%,.45);
  box-shadow: -1px 0 0 hsla(210,16%,76%,.15) inset, 1px 0 0 hsla(210,16%,76%,.15);
}

.devtools-toolbarbutton[type=menu-button]:-moz-locale-dir(rtl) > .toolbarbutton-menubutton-button {
  box-shadow: 1px 0 0 hsla(210,16%,76%,.15) inset, -1px 0 0 hsla(210,16%,76%,.15);
}

.devtools-toolbarbutton[type=menu-button] {
  padding: 0 1px;
  -moz-box-align: stretch;
}

.devtools-toolbarbutton[type=menu] > .toolbarbutton-menu-dropmarker,
.devtools-toolbarbutton[type=menu-button] > .toolbarbutton-menubutton-dropmarker {
  -moz-appearance: none !important;
  list-style-image: url("chrome://browser/skin/devtools/dropmarker.png");
  -moz-box-align: center;
  margin: 0 3px;
}

/* Search input */

.devtools-searchinput {
  -moz-appearance: none;
  margin: 0 3px;
  border: 1px solid hsla(210,8%,5%,.6);
  border-radius: 2px;
  background-color: transparent;
  background-image: url(magnifying-glass.png), -moz-linear-gradient(hsla(210,16%,76%,.15), hsla(210,16%,76%,.35));
  background-repeat: no-repeat;
  background-position: 4px center, top left, top left;
  padding-top: 0;
  padding-bottom: 0;
  -moz-padding-start: 18px;
  -moz-padding-end: 12px;
  box-shadow: 0 1px 1px hsla(210,8%,5%,.3) inset,
              0 0 0 1px hsla(210,16%,76%,.1) inset,
              0 1px 0 hsla(210,16%,76%,.15);
  color: inherit;
}

.devtools-searchinput:-moz-locale-dir(rtl) {
  background-position: calc(100% - 4px) center, top left, top left;
}

.devtools-searchinput > .textbox-input-box > .textbox-search-icons {
  display: none;
}

.devtools-searchinput > .textbox-input-box > .textbox-input:-moz-placeholder {
  color: hsl(208,10%,66%);
}

/* Close button */

.devtools-closebutton {
  list-style-image: url("moz-icon://stock/gtk-close?size=menu");
  margin-top: 0;
  margin-bottom: 0;
}

.devtools-closebutton > .toolbarbutton-icon {
  /* XXX Buttons have padding in widget/ that we don't want here but can't override with good CSS, so we must
     use evil CSS to give the impression of smaller content */
  margin: -4px;
}

/* Splitters */

.devtools-horizontal-splitter {
  -moz-appearance: none;
  border-top: 1px solid black;
  border-bottom-width: 0;
  min-height: 3px;
  height: 3px;
  margin-bottom: -3px;
  position: relative;
}

.devtools-side-splitter {
  -moz-appearance: none;
  border: 0;
  -moz-border-start: 1px solid black;
  min-width: 0;
  width: 3px;
  background-color: transparent;
  -moz-margin-end: -3px;
  position: relative;
}
