/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

ul {
  list-style: none;
}

ul.children:not([expanded]) {
  display: none;
}

.codebox {
  display: inline-block;
}

.newattr {
  display: inline-block;
  width: 1em;
  height: 1ex;
}
